@@ -59,15 +59,15 @@
 ##' ensure identifiability.)
 ##' @param index For internal use only
 ##' @param graph For internal use only
-##' @param silent Logical argument indicating whether information should be
-##' printed during estimation
+##' @param messages Control how much information should be
+##' printed during estimation (0: none)
 ##' @param quick If TRUE the parameter estimates are calculated but all
 ##' additional information such as standard errors are skipped
 ##' @param method Optimization method
 ##' @param param set parametrization (see \code{help(lava.options)})
 ##' @param cluster Obsolete. Alias for 'id'.
 ##' @param p Evaluate model in parameter 'p' (no optimization)
-##' @param ... Additional arguments to be passed to the low level functions
+##' @param ... Additional arguments to be passed to lower-level functions
 ##' @return A \code{lvmfit}-object.
 ##' @author Klaus K. Holst
 ##' @seealso estimate.default score, information
@@ -115,23 +115,22 @@
 ##' e0
 `estimate.lvm` <-
     function(x, data=parent.frame(),
-             estimator=NULL,
-             control=list(),
-             missing=FALSE,
-             weights, weightsname,
-             data2,
-             id,
-             fix,
-             index=!quick,
-             graph=FALSE,
-             silent=lava.options()$silent,
-             quick=FALSE,
-             method,
-             param,
-             cluster,
-             p,
-             ...) {
-
+      estimator=NULL,
+      control=list(),
+      missing=FALSE,
+      weights, weightsname,
+      data2,
+      id,
+      fix,
+      index=!quick,
+      graph=FALSE,
+      messages=lava.options()$messages,
+      quick=FALSE,
+      method,
+      param,
+      cluster,
+      p,
+      ...) {
         cl <- match.call()
         if (!base::missing(param)) {
             oldparam <- lava.options()$param
@@ -171,7 +170,6 @@
         if (length(control)>0) {
             Optim[names(control)] <- control
         }
-
         if (is.environment(data)) {
             innames <- intersect(ls(envir=data),vars(x))
             data <- as.data.frame(lapply(innames,function(x) get(x,envir=data)))
@@ -196,15 +194,6 @@
 
         if (!base::missing(cluster)) id <- cluster
 
-        ## commented; don't reduce data
-        ## if (!missing & (is.matrix(data) | is.data.frame(data))) {
-        ##     includelist <- c(manifest(x),xfix)
-        ##     if (!base::missing(weights) && is.character(weights)) includelist <- c(includelist,weights)
-        ##     if (!base::missing(data2) && is.character(data2)) includelist <- c(includelist,data2)
-        ##     if (!base::missing(id) && is.character(id)) includelist <- c(includelist,id)
-        ##     ##data <- na.omit(data[,intersect(colnames(data),includelist),drop=FALSE])
-        ## }
-
         ## Weights...
         if (!base::missing(weights)) {
             if (is.character(weights)) {
@@ -247,16 +236,13 @@
             var.missing <- setdiff(vars(x),colnames(data))
             S <- NULL; mu <- NULL; n <- nrow(data)
         }
-        ## Debug(list("n=",n))
-        ## Debug(list("S=",S))
-        ## Debug(list("mu=",mu))
 
         ##  if (fix) {
-            if (length(var.missing)>0) {## Convert to latent:
-                new.lat <- setdiff(var.missing,latent(x))
-                if (length(new.lat)>0)
-                    x <- latent(x, new.lat)
-            }
+        if (length(var.missing)>0) {## Convert to latent:
+            new.lat <- setdiff(var.missing,latent(x))
+            if (length(new.lat)>0)
+                x <- latent(x, new.lat)
+        }
         ##}
 
         ## Run hooks (additional lava plugins)
@@ -307,8 +293,8 @@
 
         if (index) {
             ## Proces data and setup some matrices
-            x <- fixsome(x, measurement.fix=fix, S=S, mu=mu, n=n,debug=!silent)
-            if (!silent)
+            x <- fixsome(x, measurement.fix=fix, S=S, mu=mu, n=n,debug=messages>1)
+            if (messages>1)
                 message("Reindexing model...\n")
             if (length(xfix)>0) {
                 index(x) <- reindex(x,sparse=Optim$sparse,zeroones=TRUE,deriv=TRUE)
@@ -345,7 +331,7 @@
                 if (is.null(Optim$start) || sum(paragree)<length(myparnames)) {
                     if (is.null(Optim$starterfun) && lava.options()$param!="relative")
                         Optim$starterfun <- startvalues0
-                    start <- suppressWarnings(do.call(Optim$starterfun, list(x=x,S=S,mu=mu,debug=lava.options()$debug,silent=silent,data=data,...)))
+                    start <- suppressWarnings(do.call(Optim$starterfun, list(x=x,S=S,mu=mu,debug=lava.options()$debug,messages=messages,data=data,...)))
                     if (!is.null(x$expar) && length(start)<nparall) {
                         ii <- which(index(x)$e1==1)
                         start <- c(start, structure(unlist(x$expar[ii]),names=names(x$expar)[ii]))
@@ -360,7 +346,7 @@
 
         ## Missing data
         if (missing) {
-            return(estimate.MAR(x=x,data=data,fix=fix,control=Optim,debug=lava.options()$debug,silent=silent,estimator=estimator,weights=weights,data2=data2,cluster=id,...))
+            return(estimate.MAR(x=x,data=data,fix=fix,control=Optim,debug=lava.options()$debug,messages=messages,estimator=estimator,weights=weights,data2=data2,cluster=id,...))
         }
         coefname <- coef(x,mean=Optim$meanstructure,fix=FALSE);
         names(Optim$start) <- coefname
@@ -408,7 +394,6 @@
         }
         ## Fix problems with starting values?
         Optim$start[is.nan(unlist(Optim$start))] <- 0
-        ## Debug(list("lower=",lower))
 
         ObjectiveFun  <- paste0(estimator, "_objective", ".lvm")
         GradFun  <- paste0(estimator, "_gradient", ".lvm")
@@ -435,10 +420,9 @@
                     for (j in seq_len(length(myfix$col[[i]])))
                         regfix(x0, from=vars(x0)[myfix$row[[i]][j]],
                                to=vars(x0)[myfix$col[[i]][j]]) <-
-                                   colMeans(data[,myfix$var[[i]],drop=FALSE])
+                            colMeans(data[,myfix$var[[i]],drop=FALSE])
                 x0 <- updatelvm(x0,zeroones=TRUE,deriv=TRUE)
                 x <- x0
-                yvars <- endogenous(x0)
                 ## Alter start-values/constraints:
                 new.par.idx <- which(coef(mymodel,mean=TRUE,fix=FALSE)%in%coef(x0,mean=TRUE,fix=FALSE))
                 if (length(Optim$start)>length(new.par.idx))
@@ -481,9 +465,9 @@
                     if (is.list(data2)) {
                         rr <- do.call(GradFun, list(x=x0, p=pp, data=mydata[ii,,drop=FALSE], n=1, weights=weights[ii,], data2=data2))
                     } else
-                        {
-                            rr <- do.call(GradFun, list(x=x0, p=pp, data=mydata[ii,,drop=FALSE], n=1, weights=weights[ii,], data2=data2[ii,]))
-                        }
+                    {
+                        rr <- do.call(GradFun, list(x=x0, p=pp, data=mydata[ii,,drop=FALSE], n=1, weights=weights[ii,], data2=data2[ii,]))
+                    }
                     return(rr)
                 }
                 ss <- rowSums(rbind(sapply(seq_len(nrow(data)),myfun)))
@@ -513,13 +497,13 @@
                 val <- apply(array(unlist(L),dim=c(length(pp),length(pp),nrow(data))),c(1,2),sum)
                 if (!is.null(attributes(L[[1]])$grad)) {
                     attributes(val)$grad <- colSums (
-                        matrix( unlist(lapply(L,function(i) attributes(i)$grad)) , ncol=length(pp), byrow=TRUE)
-                        )
+                                       matrix( unlist(lapply(L,function(i) attributes(i)$grad)) , ncol=length(pp), byrow=TRUE)
+                                   )
                 }
                 return(val)
             }
 
-##################################################
+            ##################################################
         } else { ## No, standard model
 
             ## Non-linear parameter constraints involving observed variables? (e.g. nonlinear regression)
@@ -550,7 +534,7 @@
                         pp <- unlist(M$parval[xconstrain[[i]]$warg]);
                         myidx <- with(xconstrain[[i]],order(c(wargidx,exoidx)))
                         D <- cbind(rbind(pp)%x%cbind(rep(1,nrow(Mu))),
-                                  data[,xconstrain[[i]]$exo,drop=FALSE])[,myidx,drop=FALSE]
+                                   data[,xconstrain[[i]]$exo,drop=FALSE])[,myidx,drop=FALSE]
                         mu <- try(xconstrain[[i]]$func(D),silent=TRUE)
                         if (is.data.frame(mu)) mu <- mu[,1]
                         if (inherits(mu,"try-error") || NROW(mu)!=NROW(Mu)) {
@@ -589,15 +573,6 @@
             myGrad <- function(pp) {
                 if (Optim$constrain)
                     pp[constrained] <- exp(pp[constrained])
-                ##  offset <- MkOffset(pp)
-                ##  mu0 <- mu; S0 <- S; x0 <- x
-                ## if (!is.null(offset)) {
-                ##   x0$constrain[iconstrain] <- NULL
-                ##   data0 <- data[,manifest(x0)]
-                ##   data0[,endogenous(x)] <- data0[,endogenous(x)]-offset
-                ##   pd <- procdata.lvm(x0,data=data0)
-                ##   S0 <- pd$S; mu0 <- pd$mu
-                ## }
                 S <- do.call(GradFun, list(x=x, p=pp, data=data, S=S, mu=mu, n=n, weights=weights
                                          , data2=data2##, offset=offset
                                            ))
@@ -655,16 +630,12 @@
         if (is.null(tryCatch(get(GradFun),error = function (x) NULL)))
             myGrad <- NULL
 
-        if (!silent) message("Optimizing objective function...")
-        if (Optim$trace>0 & !silent) message("\n")
+        if (messages>1) message("Optimizing objective function...")
+        if (Optim$trace>0 & (messages>1)) message("\n")
         ## Optimize with lower constraints on the variance-parameters
         if ((is.data.frame(data) | is.matrix(data)) && nrow(data)==0) stop("No observations")
         if (!missing(p)) {
             opt <- list(estimate=p)
-            ## if (!is.null(myGrad))
-            ##     opt <- c(opt,list(gradient=myGrad(p)))
-            ## if (!is.null(myObj))
-            ##     opt <- c(opt,list(objective=myObj(p)))
 
         } else {
             if (!is.null(Optim$method)) {
@@ -715,15 +686,14 @@
                 }
                 if (!is.null(Optim$optimx.method)) optarg$method <- Optim$optimx.method
                 opt <- do.call(Optim$method,
-                              optarg)
+                               optarg)
                 if (inherits(opt,"optimx")) {
-                    opt0 <- opt
                     opt <- list(par=coef(opt)[1,])
                 }
                 if (is.null(opt$estimate))
                     opt$estimate <- opt$par
                 if (Optim$constrain) {
-                opt$estimate[constrained] <- exp(opt$estimate[constrained])
+                    opt$estimate[constrained] <- exp(opt$estimate[constrained])
                 }
 
                 if (XconstrStdOpt & !is.null(myGrad))
@@ -743,8 +713,10 @@
         }
 
         if (!is.null(opt$convergence)) {
-            if (opt$convergence!=0) warning("Lack of convergence. Increase number of iteration or change starting values.")
-        } else if (!is.null(opt$gradient) && mean(opt$gradient)^2>1e-3) warning("Lack of convergence. Increase number of iteration or change starting values.")
+            if (opt$convergence != 0)
+                warning("Lack of convergence. Increase number of iteration or change starting values.")
+        } else if (!is.null(opt$gradient) && mean(opt$gradient)^2>1e-3)
+            warning("Lack of convergence. Increase number of iteration or change starting values.")
         if (quick) {
             return(opt$estimate)
         }
@@ -770,17 +742,17 @@
             asVar <- matrix(NA,ncol=length(pp),nrow=length(pp))
         } else {
 
-            if (!silent) message("\nCalculating asymptotic variance...\n")
+            if (messages>1) message("\nCalculating asymptotic variance...\n")
             asVarFun  <- paste0(estimator, "_variance", ".lvm")
 
             if (!exists(asVarFun)) {
                 if (is.null(myInfo)) {
-                if (!is.null(myGrad))
-                    myInfo <- function(pp)
-                        numDeriv::jacobian(myGrad,pp,method=lava.options()$Dmethod)
-                else
-                    myInfo <- function(pp)
-                        numDeriv::hessian(myObj,pp)
+                    if (!is.null(myGrad))
+                        myInfo <- function(pp)
+                            numDeriv::jacobian(myGrad,pp,method=lava.options()$Dmethod)
+                    else
+                        myInfo <- function(pp)
+                            numDeriv::hessian(myObj,pp)
                 }
                 I <- myInfo(opt$estimate)
                 asVar <- tryCatch(Inverse(I),
@@ -803,18 +775,17 @@
         mycoef <- matrix(NA,nrow=nparall,ncol=4)
         mycoef[pp.idx,1] <- opt$estimate ## Will be finished during post.hooks
 
-### OBS: v = t(A)%*%v + e
         res <- list(model=x, call=cl, coef=mycoef,
-                   vcov=asVar, mu=mu, S=S, ##A=A, P=P,
-                   model0=mymodel, ## Random slope hack
-                   estimator=estimator, opt=opt,expar=x$expar,
-                   data=list(model.frame=data, S=S, mu=mu,
-                             C=mom$C, v=mom$v, n=n,
-                             m=length(latent(x)), k=length(index(x)$manifest), data2=data2),
-                   weights=weights, data2=data2,
-                   cluster=id,
-                   pp.idx=pp.idx,
-                   graph=NULL, control=Optim)
+                    vcov=asVar, mu=mu, S=S, ##A=A, P=P,
+                    model0=mymodel, ## Random slope hack
+                    estimator=estimator, opt=opt,expar=x$expar,
+                    data=list(model.frame=data, S=S, mu=mu,
+                              C=mom$C, v=mom$v, n=n,
+                              m=length(latent(x)), k=length(index(x)$manifest), data2=data2),
+                    weights=weights, data2=data2,
+                    cluster=id,
+                    pp.idx=pp.idx,
+                    graph=NULL, control=Optim)
 
         class(res) <- myclass
 
@@ -834,50 +805,3 @@
 
 ###}}} estimate.lvm
 
-###{{{ estimate.formula
-
-##' @export
-estimate.formula <- function(x,data=parent.frame(),pred.norm=c(),unstruct=FALSE,silent=TRUE,id=NULL,distribution=NULL,estimator="gaussian",...) {
-    cl <- match.call()
-    formulaId <- union(Specials(x,c("cluster")),Specials(x,c("id")))
-    formulaSt <- paste0("~.-cluster(",formulaId,")-id(",formulaId,")")
-    if (!is.null(formulaId)) {
-        id <- formulaId
-        x <- update(x,as.formula(formulaSt))
-    }
-    if (!is.null(id))
-        x <- update(x,as.formula(paste(".~.+",id)))
-    varnames <- all.vars(x)
-    mf <- model.frame(x,data)
-    mt <- attr(mf, "terms")
-    yvar <- names(mf)[1]
-    y <- mf[,yvar]
-    opt <- options(na.action="na.pass")
-    mm <- model.matrix(x,data)
-    options(opt)
-    covars <- colnames(mm)
-    covars <- unlist(lapply(covars, function(x) gsub("[^a-zA-Z0-9._]","",x)))
-    colnames(mm) <- covars
-
-    if (attr(terms(x),"intercept")==1) {
-        covars <- covars[-1]
-        it <- c()
-    } else {
-        it <- "0"
-    }
-
-    if (!is.null(id)) covars <- setdiff(covars,id)
-    model <- lvm(toformula(yvar,c(it,covars)),silent=TRUE)
-    if (!is.null(distribution)) {
-        lava::distribution(model,yvar) <- distribution
-        estimator <- "glm"
-    }
-    mydata <- na.omit(as.data.frame(cbind(data.frame(y),mm))); names(mydata)[1] <- yvar
-    exogenous(model) <- setdiff(covars,pred.norm)
-    if (unstruct) {
-        model <- covariance(model,pred.norm,pairwise=TRUE)
-    }
-    estimate(model,mydata,silent=silent,id=id,estimator=estimator,...)
-}
-
-###}}} estimate.formula
