@@ -13,6 +13,7 @@
 ##' binomial.lvm
 ##' poisson.lvm
 ##' uniform.lvm
+##' multinomial.lvm
 ##' beta.lvm
 ##' normal.lvm mvn.lvm
 ##' lognormal.lvm
@@ -345,7 +346,6 @@ sim.lvm <- function(x,n=NULL,p=NULL,normal=FALSE,cond=FALSE,sigma=1,rho=.5,
         mu <- unlist(lapply(x$mean, function(l) ifelse(is.na(l)|is.character(l),0,l)))
         xf <- intersect(unique(parlabels(x)),xx)
         xfix <- c(randomslope(x),xf); if (length(xfix)>0) normal <- FALSE
-
         ## Match parameter names
         if ((!is.null(names(p)) && all(!is.na(names(p)))) || length(p)!=(index(x)$npar+index(x)$npar.mean+index(x)$npar.ex) | is.null(names(p))) {
             nullp <- is.null(p)
@@ -365,13 +365,12 @@ sim.lvm <- function(x,n=NULL,p=NULL,normal=FALSE,cond=FALSE,sigma=1,rho=.5,
                 idx11 <- na.omit(match(names(p0),c2))
                 idx2 <- na.omit(which(names(p0)%in%c1))
                 idx22 <- na.omit(which(names(p0)%in%c2))
-                if (length(idx1)>0 && !is.na(idx1))
+                if (length(idx1)>0 && !any(is.na(idx1)))
                     p[idx1] <- p0[idx2]
-                if (length(idx11)>0 && !is.na(idx11))
+                if (length(idx11)>0 && !any(is.na(idx11)))
                     p[idx11] <- p0[idx22]
             }
         }
-
         M <- modelVar(x,p,data=NULL)
         A <- M$A; P <- M$P
         if (!is.null(M$v)) mu <- M$v
